import React__default, { useRef, useEffect, useReducer, useCallback, forwardRef, createElement, memo, useContext, Fragment } from 'react';

// globals only for Shuttle.Container
const NUMBER_OF_CONTAINERS = 2;
const SHUTTLE_CONTAINERS_ARRAY = ['source', 'target'];
/**
 * Global IDs of the containers. This allows us to quickly reference the containers
 * in the DOM in the event `getItemProps()` is not passed to `Shuttle.Item`.
 */
var SHUTTLE_CONTAINERS;
(function (SHUTTLE_CONTAINERS) {
    SHUTTLE_CONTAINERS["SOURCE"] = "source";
    SHUTTLE_CONTAINERS["TARGET"] = "target";
})(SHUTTLE_CONTAINERS || (SHUTTLE_CONTAINERS = {}));

/**
 * A contrived implementation of classNames. This allows
 * react-accessible-shuttle to have zero dependencies.
 */
function classNames(...args) {
    let selectors = '';
    for (let i = 0; i < args.length; i++) {
        if (typeof args[i] === 'string') {
            selectors += ` ${args[i]}`;
        }
        else if (Array.isArray(args[i])) {
            selectors += args[i].join(' ');
        }
        else if (typeof args[i] === 'object') {
            selectors += ' ';
            selectors += Object.keys(args[i])
                .filter(key => {
                if (args[i][key]) {
                    return args[i][key];
                }
            })
                .join(' ');
        }
    }
    return selectors.trim();
}
/**
 * Small warn wrapper.
 * @see https://github.com/i18next/react-i18next/blob/master/src/utils.js
 */
function warn(...args) {
    if (console && console.warn) {
        if (typeof args[0] === 'string') {
            args[0] = `react-shuttle:: ${args[0]}`;
        }
        console.warn(...args);
    }
}
const alreadyWarned = {};
/**
 * Warn about issues only once.
 * @see https://github.com/i18next/react-i18next/blob/master/src/utils.js
 */
function warnOnce(...args) {
    if (typeof args[0] === 'string' && alreadyWarned[args[0]]) {
        return;
    }
    if (typeof args[0] === 'string') {
        alreadyWarned[args[0]] = new Date();
    }
    warn(...args);
}
/**
 * Converts the source array to a Set.
 * Needed because IE 11 fails with `new Set([1,2,3])`.
 */
function toSet(source) {
    const set = new Set();
    source.forEach(item => set.add(item));
    return set;
}
/**
 * Filters only visible items. This happens when the selection array is
 * greater than the collection length.
 * @see removeDisabledIndexes for usage
 */
const filterVisibleItems = (collection, indexes) => {
    const set = toSet(indexes);
    const matches = [];
    for (let i = 0; i < collection.length; i++) {
        const index = collection[i].getAttribute('data-index');
        const disabled = collection[i].hasAttribute('data-disabled');
        if (!disabled && index && set.has(parseInt(index))) {
            matches.push(parseInt(index));
        }
    }
    return matches;
};
/**
 * Gets the index of the HTMLElement. As an escape hatch, if `data-index` was not passed
 * to the Shuttle.Item we will look through the list to find the item we want.
 */
const getIndexFromItem = (target) => {
    if (!target.hasAttribute('data-index')) {
        warnOnce('Did you forget to pass getItemProps on each Shuttle.Item? This is sever impact on performance.');
        const container = target.closest('.shuttle__container');
        if (container) {
            let node = target;
            let index = 0;
            while ((node = node.previousElementSibling)) {
                index++;
            }
            return index;
        }
        return -1;
    }
    const index = parseInt(target.getAttribute('data-index') || '', 10);
    return isNaN(index) ? -1 : index;
};
/**
 * Gets the shuttle item from the DOM since consumers can render whatever
 * they want inside the item itself, we might need to look up the DOM tree
 * to get the HTMLElement.
 */
const getShuttleItem = (e) => e.closest('.shuttle__item');
const removeDisabledIndexes = (collection, indexes) => {
    if (collection.length < indexes.length) {
        return filterVisibleItems(collection, indexes);
    }
    // FIXME: DOM nodes should not hold state. If the disabled items needs to be updated
    // consumers should update the set themselves
    return indexes.filter(index => index >= 0 &&
        index < collection.length &&
        !collection[index].hasAttribute('data-disabled'));
};
/**
 * Gets relevant metadata from the container including
 * if the container is the source or target and the name
 * of the container.
 */
const getContainerMetadata = (container) => {
    const source = container.getAttribute('data-name') === SHUTTLE_CONTAINERS.SOURCE;
    const containerName = (source && SHUTTLE_CONTAINERS.SOURCE) || SHUTTLE_CONTAINERS.TARGET;
    return { source, containerName };
};
const isContainer = (container) => {
    const bool = container.hasAttribute('data-name');
    const name = container.getAttribute('data-name');
    return bool && (name === SHUTTLE_CONTAINERS.SOURCE || name === SHUTTLE_CONTAINERS.TARGET);
};
/**
 * @see https://github.com/then/is-promise/blob/master/index.js
 */
function isPromise(obj) {
    return (!!obj &&
        (typeof obj === 'object' || typeof obj === 'function') &&
        typeof obj.then === 'function');
}

const shuttleSelections = (from, to, selected) => {
    const entries = selected.entries();
    for (const [entry] of entries) {
        to.push(from.splice(entry, 1, null)[0]);
    }
    return from.filter(Boolean);
};
const move = (state, action = {}) => {
    if (action.type === 'MOVE_SELECTIONS') {
        if (!action.from || !action.to) {
            throw new Error(`Missing required actions: from: 'source' | 'target', to: 'source' | 'target'`);
        }
        state[action.from] = shuttleSelections(state[action.from], state[action.to], state.selected[action.from]);
        state.selected[action.from].clear();
        return { ...state };
    }
    return { ...state };
};

/**
 * Compact an array safely and in-place. This function works
 * in conjunction with `shuttleAll`.
 */
const compact = (list, source) => {
    // spread is overwhelmingly faster https://jsperf.com/array-allocation-perf/1
    const result = [...Array(list.length)];
    let pointer = 0;
    for (let i = 0; i < list.length; i++) {
        if (source[list[i]] !== null) {
            result[pointer++] = source[list[i]];
        }
    }
    return result;
};
/**
 * Shuttle all items from one array to another array. Specify blacklisted item indexes
 * to prevent those from being moved.
 */
const shuttleAll = (from, to, disabled) => {
    // this is way more optimal if there are no disabled items
    if (!disabled.size) {
        Array.prototype.push.apply(to, from);
        from.length = 0;
    }
    else if (disabled.size < from.length) {
        const disabledIndexes = [];
        for (let i = 0; i < from.length; i++) {
            if (!disabled.has(from[i])) {
                to.push(from.splice(i, 1, null)[0]);
            }
            else {
                disabledIndexes.push(i);
            }
        }
        return compact(disabledIndexes, from);
    }
    return from;
};
const moveAll = (state, action = {}) => {
    if (action.type === 'MOVE_ALL') {
        if (!action.from || !action.to) {
            throw new Error(`Missing required reducer properties: { from: 'source' | 'target', to: 'source' | 'target' }`);
        }
        state[action.from] = shuttleAll(state[action.from], state[action.to], state.disabled[action.from]);
        state.selected[action.from].clear();
        return { ...state };
    }
    return { ...state };
};

const selectItem = (state, action) => {
    if (action.type === 'SELECT_ITEM') {
        if ((typeof action.index === 'number' || Array.isArray(action.index)) &&
            !action.container) {
            throw new Error(`'container' is required when items are passed to the reducer`);
        }
        state.selected[action.container].clear();
        if (Array.isArray(action.index)) {
            action.index.forEach(i => {
                state.selected[action.container].add(i);
            });
        }
        else if (typeof action.index === 'number') {
            state.selected[action.container].add(action.index);
        }
    }
    return { ...state };
};

const lazyLoad = (state, action) => {
    if (action.type === 'LAZY_LOAD') {
        return init(action.data);
    }
    return { ...state };
};

var SHUTTLE_CONTROL_TYPES;
(function (SHUTTLE_CONTROL_TYPES) {
    SHUTTLE_CONTROL_TYPES["MOVE_SELECTIONS"] = "MOVE_SELECTIONS";
    SHUTTLE_CONTROL_TYPES["MOVE_ALL"] = "MOVE_ALL";
    SHUTTLE_CONTROL_TYPES["SELECT_ITEM"] = "SELECT_ITEM";
})(SHUTTLE_CONTROL_TYPES || (SHUTTLE_CONTROL_TYPES = {}));
/**
 * Redux-style composeReducers function. The main difference here
 * is that this function operates on the _same_ slice of state
 * for _all_ reducer functions.
 * @param reducers An object of key: function pairs.
 * @see useShuttleState for usage and other configuration docs.
 */
const composeReducers = (reducers) => (state, action) => {
    const result = { ...state };
    for (const key in reducers) {
        if (typeof reducers[key] !== 'function') {
            throw new Error(`Reducers must be functions. Saw ${typeof reducers[key]} for ${key}`);
        }
        Object.assign(result, reducers[key](result, action));
    }
    return { ...result };
};

/**
 * An internal hook to lazily set the state from a Promise
 * or something that is Promise A+ compliant. We really only
 * want this hook to run when the state is first initialized.
 */
function useAsyncState(state, setShuttleState) {
    const resolved = useRef(false);
    useEffect(() => {
        if (isPromise(state) && !resolved.current) {
            state.then(data => {
                resolved.current = true;
                setShuttleState({ type: 'LAZY_LOAD', data });
            });
        }
    }, []); // eslint-disable-line react-hooks/exhaustive-deps
}

function init({ source = [], target = [], selections = {
    source: [],
    target: [],
}, disabled = {
    source: [],
    target: [],
}, }) {
    if (!Array.isArray(selections.source)) {
        throw new Error('Initial selection "source" must be an array');
    }
    if (!Array.isArray(selections.target)) {
        throw new Error('Initial selection "target" must be an array');
    }
    return {
        source,
        target,
        selected: {
            source: toSet(selections.source),
            target: toSet(selections.target),
        },
        disabled: {
            source: toSet(disabled.source),
            target: toSet(disabled.target),
        },
    };
}
/**
 * useShuttleState allows you to own the state of the Shuttle. Pass in custom
 * data without trying to remember what prop needs to be set and with what params.
 * Save this result in a variable and spread ot back to `<Shuttle>`. Let the Shuttle
 * know:
 *
 * 1. What your data is
 * 2. What (if any) items need to be pre-selected
 * 3. What (if any) custom state reducers you want to process -- you can even override Shuttle state reducers here too!
 * @param initialState
 * @param initialSelections
 * @param reducers
 */
function useShuttleState(initialState = {
    source: [],
    target: [],
}, initialSelections, disabled, reducers = {}) {
    const [shuttleState, setShuttleState] = useReducer(useCallback(composeReducers({ move, moveAll, selectItem, lazyLoad, ...reducers }), []), {
        ...initialState,
        // @ts-ignore
        disabled: disabled || {
            source: [],
            target: [],
        },
        selections: initialSelections || {
            source: [],
            target: [],
        },
    }, init);
    useAsyncState(initialState, setShuttleState);
    return { shuttleState, setShuttleState };
}

function useShuttleItemClick({ setShuttleState }) {
    const onClick = useCallback((e) => {
        const target = getShuttleItem(e.target);
        if (target && target.className.includes('shuttle__item')) {
            const index = getIndexFromItem(target);
            const container = target.closest('.shuttle__container');
            if (container) {
                const { source } = getContainerMetadata(container);
                const action = {
                    type: SHUTTLE_CONTROL_TYPES.SELECT_ITEM,
                    container: source ? 'source' : 'target',
                    index,
                };
                setShuttleState(action);
            }
        }
    }, [setShuttleState]);
    return { onClick };
}

var KEYS;
(function (KEYS) {
    KEYS[KEYS["END"] = 35] = "END";
    KEYS[KEYS["HOME"] = 36] = "HOME";
    KEYS[KEYS["UP_ARROW"] = 38] = "UP_ARROW";
    KEYS[KEYS["DOWN_ARROW"] = 40] = "DOWN_ARROW";
})(KEYS || (KEYS = {}));
const handleShiftKeyboardControl = (selectionArray, increment, container) => {
    const lastItemInArray = selectionArray[selectionArray.length - 1];
    if (increment) {
        if (selectionArray.indexOf(lastItemInArray - 1) === -1) {
            selectionArray.push(lastItemInArray - 1);
        }
        else {
            selectionArray.pop();
        }
    }
    else {
        if (selectionArray.indexOf(lastItemInArray + 1) === -1) {
            selectionArray.push(lastItemInArray + 1);
        }
        else {
            selectionArray.pop();
        }
    }
    selectionArray = removeDisabledIndexes(container.children, selectionArray);
    return selectionArray;
};
const handleDefaultKeyboardControl = (selectionArray, increment, shuttleItemParent) => {
    let [index] = selectionArray;
    index = index + (increment ? -1 : 1);
    while (index >= 0 &&
        index < shuttleItemParent.children.length &&
        shuttleItemParent.children[index].hasAttribute('data-disabled')) {
        index = index + (increment ? -1 : 1);
    }
    return index;
};
function useShuttleKeyboardControls({ setShuttleState, shuttleState, useMeta = true, useShift = true, }) {
    const shiftKeyPressed = useRef(false);
    const ctrlKeyPressed = useRef(false);
    const metaKeyPressed = useRef(false);
    const { onClick: defaultClickHandler } = useShuttleItemClick({ setShuttleState, shuttleState });
    const onKeyDown = useCallback((e) => {
        if (useShift) {
            shiftKeyPressed.current = e.shiftKey;
        }
        if (useMeta) {
            ctrlKeyPressed.current = e.ctrlKey;
            metaKeyPressed.current = e.metaKey;
        }
        if (e.keyCode === KEYS.UP_ARROW || e.keyCode === KEYS.DOWN_ARROW) {
            e.preventDefault();
        }
    }, [useMeta, useShift]);
    const onKeyUp = useCallback((e) => {
        if (useShift) {
            shiftKeyPressed.current = e.shiftKey;
        }
        if (useMeta) {
            ctrlKeyPressed.current = e.ctrlKey;
            metaKeyPressed.current = e.metaKey;
        }
        // TODO: implement HOME and END keycodes support
        // https://www.w3.org/TR/wai-aria-practices-1.1/examples/listbox/listbox-scrollable.html
        if (e.keyCode === KEYS.UP_ARROW || e.keyCode === KEYS.DOWN_ARROW) {
            e.preventDefault();
            const item = getShuttleItem(e.target);
            if (isContainer(e.target)) {
                const container = e.target;
                const { containerName } = getContainerMetadata(container);
                setShuttleState({
                    type: 'SELECT_ITEM',
                    container: containerName,
                    index: 0,
                });
                container.children[0].focus();
            }
            else if (item && item.className.includes('shuttle__item')) {
                const itemIndex = getIndexFromItem(item);
                const increment = e.keyCode === KEYS.UP_ARROW;
                const container = item.closest('.shuttle__container');
                if (container) {
                    // dereference the parent to shuttle__item in th event we're wrapping
                    // or virtualizing the container
                    const shuttleItemParent = item.parentElement || container;
                    const { containerName } = getContainerMetadata(container);
                    if (itemIndex >= 0 && itemIndex < shuttleState[containerName].length) {
                        const selectionArray = Array.from(shuttleState.selected[containerName]);
                        const payload = {
                            type: 'SELECT_ITEM',
                            container: containerName,
                        };
                        if (shiftKeyPressed.current) {
                            payload.index = handleShiftKeyboardControl(selectionArray, increment, container);
                            shuttleItemParent.children[payload.index[payload.index.length - 1]].focus();
                        }
                        else {
                            payload.index = handleDefaultKeyboardControl(selectionArray, increment, shuttleItemParent);
                            if (payload.index < 0 ||
                                payload.index >= shuttleItemParent.children.length) {
                                return;
                            }
                            shuttleItemParent.children[payload.index].focus();
                        }
                        setShuttleState(payload);
                    }
                }
            }
        }
    }, [setShuttleState, shuttleState, useMeta, useShift]);
    const onClick = useCallback((e) => {
        if (ctrlKeyPressed.current || metaKeyPressed.current || shiftKeyPressed.current) {
            const target = getShuttleItem(e.target);
            if (target) {
                const index = getIndexFromItem(target);
                const container = target.closest('.shuttle__container');
                if (container) {
                    const { containerName } = getContainerMetadata(container);
                    const payload = {
                        type: 'SELECT_ITEM',
                        container: containerName,
                        index: Array.from(shuttleState.selected[containerName]),
                    };
                    let selectionArray = Array.from(shuttleState.selected[containerName]);
                    const lastItemInArray = selectionArray[selectionArray.length - 1];
                    if (shiftKeyPressed.current) {
                        if (index < lastItemInArray) {
                            let i = lastItemInArray;
                            while (i-- > index) {
                                selectionArray.push(i);
                            }
                        }
                        else if (index > lastItemInArray) {
                            let i = lastItemInArray;
                            while (i++ < index) {
                                selectionArray.push(i);
                            }
                        }
                        selectionArray = removeDisabledIndexes((target.parentElement && target.parentElement.children) ||
                            container.children, selectionArray);
                    }
                    else {
                        const foundIndex = selectionArray.indexOf(index);
                        if (foundIndex === -1) {
                            selectionArray.push(index);
                        }
                        else {
                            selectionArray.splice(foundIndex, 1);
                        }
                    }
                    payload.index = selectionArray;
                    setShuttleState(payload);
                }
            }
        }
        else {
            defaultClickHandler(e);
        }
    }, [defaultClickHandler, setShuttleState, shuttleState.selected]);
    return { onKeyDown, onKeyUp, onClick };
}

const ShuttleItem = forwardRef(({ children, className, disabled, value, selected, ...rest }, ref) => {
    if (typeof rest['data-index'] !== 'number') {
        warnOnce('data-index is missing from Shuttle.Item. Did you forget to pass getItemProps to the item?');
    }
    return (createElement("div", Object.assign({ className: classNames('shuttle__item', {
            'shuttle__item--disabled': disabled,
            'shuttle__item--selected': selected,
        }, className), "data-value": value, "data-disabled": disabled || undefined, role: "option", "aria-selected": !!selected, tabIndex: -1, ref: ref }, rest), children));
});
ShuttleItem.displayName = 'Shuttle.Item';

// @ts-ignore
const ShuttleContext = React__default.createContext({});

let id_int = 0; // eslint-disable-line @typescript-eslint/camelcase
/**
 * Pass a child render function or a render prop.
 */
const ShuttleContainer = memo(
// eslint-disable-next-line react/display-name
forwardRef(({ children, className, ...rest }, ref) => {
    const { shuttleState } = useContext(ShuttleContext);
    // mod needed for HMR updates
    const id = useRef(SHUTTLE_CONTAINERS_ARRAY[Math.floor(id_int++ % NUMBER_OF_CONTAINERS)] // eslint-disable-line @typescript-eslint/camelcase
    );
    /**
     * Pass the props to the item as you render it.
     * This is important to include since it contains
     * optimizations for click events on the shuttle
     * item.
     */
    const getItemProps = useCallback((index) => {
        return {
            'data-index': index,
            selected: shuttleState.selected[id.current].has(index),
        };
    }, [shuttleState]);
    return (createElement("div", Object.assign({ className: classNames('shuttle__container', className), role: "listbox" }, rest, { "data-name": id.current, ref: ref, tabIndex: 0 }), children(shuttleState, getItemProps)));
}));
ShuttleContainer.displayName = 'Shuttle.Container';

var CONTROL_BUTTONS;
(function (CONTROL_BUTTONS) {
    CONTROL_BUTTONS["MOVE_ALL_TARGET"] = "MOVE_ALL_TARGET";
    CONTROL_BUTTONS["MOVE_SELECTED_TARGET"] = "MOVE_SELECTED_TARGET";
    CONTROL_BUTTONS["MOVE_SELECTED_SOURCE"] = "MOVE_SELECTED_SOURCE";
    CONTROL_BUTTONS["MOVE_ALL_SOURCE"] = "MOVE_ALL_SOURCE";
})(CONTROL_BUTTONS || (CONTROL_BUTTONS = {}));
const getButtonProps = (button, size) => {
    const props = {
        moveAllToTarget: {
            title: 'Move All Items to Target',
            'aria-label': `Move All Items to Target`,
        },
        moveSelectedToTarget: {
            title: 'Move Selected Items to Target',
            'aria-label': `Move Selected Items to Target (${size} items selected in source container)`,
        },
        moveSelectedToSource: {
            title: 'Move Selected Items to Source',
            'aria-label': `Move Selected Items to Source (${size} items selected in target container)`,
        },
        moveAllToSource: {
            title: 'Move All Items to Source',
            'aria-label': `Move All Items to Source`,
        },
    };
    switch (button) {
        case CONTROL_BUTTONS.MOVE_ALL_TARGET:
            return props.moveAllToTarget;
        case CONTROL_BUTTONS.MOVE_SELECTED_TARGET:
            return props.moveSelectedToTarget;
        case CONTROL_BUTTONS.MOVE_SELECTED_SOURCE:
            return props.moveSelectedToSource;
        case CONTROL_BUTTONS.MOVE_ALL_SOURCE:
            return props.moveAllToSource;
    }
    return {};
};
/**
 * ShuttleControls. Provide your own render function
 * to customize the buttons. The Shuttle `dispatch` method
 * is provided or you can use your own state reducer
 * as needed.
 *
 * @example
 * // Custom render controls
 * <Shuttle.Controls render={setShuttleState => (
 *      <CustomButton onClick={() => setShuttleState({ type: 'MOVE_ALL' })} />
 * )} />
 */
const ShuttleControls = memo(({ children, className, ...rest }) => {
    const { setShuttleState, shuttleState } = useContext(ShuttleContext);
    const moveAllFromSource = useCallback(() => {
        setShuttleState({
            type: SHUTTLE_CONTROL_TYPES.MOVE_ALL,
            from: 'source',
            to: 'target',
        });
    }, [setShuttleState]);
    const moveAllFromTarget = useCallback(() => {
        setShuttleState({
            type: SHUTTLE_CONTROL_TYPES.MOVE_ALL,
            from: 'target',
            to: 'source',
        });
    }, [setShuttleState]);
    const moveSelectedFromSource = useCallback(() => {
        setShuttleState({
            type: SHUTTLE_CONTROL_TYPES.MOVE_SELECTIONS,
            from: 'source',
            to: 'target',
        });
    }, [setShuttleState]);
    const moveSelectedFromTarget = useCallback(() => {
        setShuttleState({
            type: SHUTTLE_CONTROL_TYPES.MOVE_SELECTIONS,
            from: 'target',
            to: 'source',
        });
    }, [setShuttleState]);
    return (createElement("div", Object.assign({ className: classNames('shuttle__controls', className) }, rest), typeof children === 'function' ? (children({
        setShuttleState,
        getButtonProps,
        moveAllFromSource,
        moveSelectedFromSource,
        moveSelectedFromTarget,
        moveAllFromTarget,
    })) : (createElement(Fragment, null,
        createElement("button", Object.assign({}, getButtonProps(CONTROL_BUTTONS.MOVE_ALL_TARGET), { onClick: moveAllFromSource }), '\u00BB'),
        createElement("button", Object.assign({}, getButtonProps(CONTROL_BUTTONS.MOVE_SELECTED_TARGET, shuttleState.selected.source.size), { onClick: moveSelectedFromSource }), '\u203A'),
        createElement("button", Object.assign({}, getButtonProps(CONTROL_BUTTONS.MOVE_SELECTED_SOURCE, shuttleState.selected.source.size), { onClick: moveSelectedFromTarget }), '\u2039'),
        createElement("button", Object.assign({}, getButtonProps(CONTROL_BUTTONS.MOVE_ALL_SOURCE), { onClick: moveAllFromTarget }), '\u00AB')))));
});
ShuttleControls.displayName = 'Shuttle.Controls';

const Shuttle = ({ shuttleState, setShuttleState, enableUserSelectionHack = true, className, children, ...rest }) => {
    const { onClick: defaultClickHandler } = useShuttleItemClick({ setShuttleState, shuttleState });
    return (createElement(ShuttleContext.Provider, { value: { shuttleState, setShuttleState } },
        createElement("div", Object.assign({ className: classNames('shuttle', {
                'shuttle--ush': enableUserSelectionHack,
            }, className), role: "presentation", onClick: defaultClickHandler }, rest), children)));
};
Shuttle.Item = ShuttleItem;
Shuttle.Container = ShuttleContainer;
Shuttle.Controls = ShuttleControls;

const __CLIENT__ = !!(typeof window !== 'undefined' &&
    window.document &&
    window.document.createElement);
function matchPolyfill() {
    if (typeof Element.prototype.matches !== 'function') {
        // https://developer.mozilla.org/en-US/docs/Web/API/Element/matches#Polyfill
        Element.prototype.matches = Element.prototype.matches =
            // @ts-ignore
            Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector;
    }
}
function closestPolyfill() {
    if (typeof Element.prototype.closest !== 'function') {
        // https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#Polyfill
        Element.prototype.closest = function closest(selector) {
            let element = this;
            do {
                if (element.matches(selector)) {
                    return element;
                }
                element = element.parentElement || element.parentNode;
            } while (element !== null && element.nodeType === 1);
            return null;
        };
    }
}
function polyfill() {
    if (__CLIENT__) {
        matchPolyfill();
        closestPolyfill();
    }
}

/* istanbul ignore file */
polyfill();

export { Shuttle, useShuttleKeyboardControls, useShuttleState };
//# sourceMappingURL=index.js.map
